# -*- coding: ascii -*-
#
#  logs.py - Log manipulator for GBottler
#  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
#  Copyright (C) 2004 by Atzm WATANABE <sitosito@p.chan.ne.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#
# $Id: logs.py,v 1.8 2004/04/25 07:09:42 atzm Exp $
#

import os, sys, string, re, time
from types import *

if os.name == 'posix':
	import fcntl

import textmanager
from bottleparser import BottleParser

import gtk

class LogManager:
	DBFILE = "log_db"
	DBWRITEFREQ = 10 # write database to file per 10 votes
	# channel
	ALL_CHANNELS = "-"
	ALL_GHOSTS   = "-"
	# ranges
	RANGE_TODAY = 1
	RANGE_YESTERDAY = 2
	RANGE_LAST3DAYS = 3
	RANGE_LAST7DAYS = 4
	RANGE_SPECIFIED = 5
	RANGE_ALL = 6
	RANGE_LAST50BOTTLES = 7
	# orders
	ORDER_BY_NEWNESS  = 100
	ORDER_BY_DATETIME = 101
	ORDER_BY_VOTES	= 102

	def __init__(self, vote, agree):
		self.vote = vote
		self.agree = agree

		from gbottler import open_bottlecase
		self.logdir = open_bottlecase()
		# lock database
		self.lockfile = open(os.path.join(self.logdir, ".lock"), "w")
		self.disabled = 0
		if os.name == 'posix':
			try:
				fcntl.flock(self.lockfile.fileno(), fcntl.LOCK_EX|fcntl.LOCK_NB)
			except IOError:
				self.disabled = 1
				return
		# load database
		self.dbpath = os.path.join(self.logdir, self.DBFILE)
		self.dblife = self.DBWRITEFREQ
		self.load_database()
		# defaults
		self.channel = None # all channels
		self.ghost   = None
		self.range   = self.RANGE_TODAY
		self.order   = self.ORDER_BY_NEWNESS

	def is_disabled(self):
		return self.disabled

	def load_database(self):
		self.aliases = {}
		self.votes = {}
		try:
			file = open(self.dbpath)
		except IOError:
			pass
		else:
			nchannels = int(file.readline())
			for n in range(nchannels):
				channel, alias = string.split(file.readline())
				channel = unicode(channel, "utf-8")
				self.aliases[channel] = alias
			while 1:
				line = file.readline()
				if not line:
					break
				record = string.split(line)
				try:
					dict = self.votes[record[0]]
				except KeyError:
					dict = self.votes[record[0]] = {}
				dict[self.vote] = int(record[1])
				if len(record) == 2:
					dict[self.agree] = 0
				else:
					dict[self.agree] = int(record[2])
		self.channels = self.aliases.keys()
		self.channels.sort()

	def save_database(self):
		try:
			file = open(self.dbpath, "w")
		except IOError, (errno, message):
			sys.stderr.write("Error: cannot write %s\n" % self.dbpath)
		else:
			file.write("%d\n" % len(self.channels))
			for channel, alias in self.aliases.items():
				file.write("%s %s\n" % (channel.encode("utf-8"), alias))
			for mid, dict in self.votes.items():
				v = dict.get(self.vote,  0)
				a = dict.get(self.agree, 0)
				file.write("%s %d %d\n" % (mid, v, a))
			file.close()

	def close(self):
		if self.disabled:
			return
		self.save_database()
		if os.name == 'posix':
			fcntl.flock(self.lockfile.fileno(), fcntl.LOCK_UN)

	def log_message(self, mid, channel, ifghost, script):
		if self.disabled:
			return
		# register an alias to channel if necessary

		if channel not in self.channels:
			self.channels.append(channel)
			self.channels.sort()
			self.aliases[channel] = "ch-%d" % len(self.channels)
		# write message to log file
		now = time.localtime(time.time())
		datetime = time.strftime("%Y%m%d%H%M%S", now)
		filename = "%s-%s.log" % (datetime[:8], self.aliases[channel])
		file = open(os.path.join(self.logdir, filename), "a")
		file.write("%s\t%s\t%s\t%s\t%s\n" % (datetime, mid.encode("utf-8"),
											 channel.encode("utf-8"),
											 ifghost.encode("utf-8"),
											 script.encode("utf-8")))
		file.close()

	def log_votes(self, mid, type, num):
		if self.disabled:
			return
		try:
			dict = self.votes[mid]
		except KeyError:
			dict = self.votes[mid] = {}
		dict[type] = num
		self.dblife = self.dblife - 1
		if self.dblife == 0:
			self.dblife = self.DBWRITEFREQ
			self.save_database()

	def read_log(self, path): # internal
		buffer = []
		file = open(path)
		while 1:
			line = unicode(file.readline(), "utf-8")
			if not line:
				break
			datetime, mid, channel, ifghost, script = string.split(line, "\t", 4)
			assert(len(datetime) == 14)
			dict = self.votes.get(mid, {})
			v = dict.get(self.vote,  0)
			a = dict.get(self.agree, 0)
			buffer.append((datetime, mid, v, a,
						   channel, ifghost, string.rstrip(script)))
		return buffer

	def reload(self):
		if self.disabled:
			return 1
		# check range
		now = time.localtime(time.time())
		e = time.mktime(now[0:3] + (0, 0, 0, 0, 0, 0)) + 86400
		s = None
		if self.range == self.RANGE_TODAY:
			s = e - 86400
		elif self.range == self.RANGE_YESTERDAY:
			s = e - 86400 * 2
			e = e - 86400 * 2
		elif self.range == self.RANGE_LAST3DAYS:
			s = e - 86400 * 3
		elif self.range == self.RANGE_LAST7DAYS:
			s = e - 86400 * 7
		elif self.range == self.RANGE_SPECIFIED:
			s = self.range_start
			e = self.range_end
		elif self.range in [self.RANGE_ALL, self.RANGE_LAST50BOTTLES]:
			s = 0
		range_start = time.strftime("%Y%m%d", time.localtime(s))
		range_end   = time.strftime("%Y%m%d", time.localtime(e))
		# read log files
		self.log = []
		self.selected_channels = []
		channels = {} # alias to channel
		for key, value in self.aliases.items():
			channels[value] = key

		list = os.listdir(self.logdir)
		list.sort()
		list.reverse()
		dateline = int(range_end) - 1
		for filename in list:
			# filename should be in the form of YYYYMMDD-CHANNEL.log
			if filename[-4:] == ".log":

				# Hmm... fixme
				if self.range == self.RANGE_LAST50BOTTLES and \
					   not int(filename[:8]) == dateline and \
					   len(self.log) >= 50:
					break

				if range_start <= filename[:8] <= range_end:
					alias = filename[9:-4]
					channel = channels.get(alias)
					if channel and channel not in self.selected_channels:
						self.selected_channels.append(channel)
					if self.channel == self.ALL_CHANNELS or \
						   self.aliases[self.channel] == alias:
						path = os.path.join(self.logdir, filename)
						self.log.extend(self.read_log(path))

				dateline = int(filename[:8])

		self.selected_channels.sort()
		# sort logs
		if self.order == self.ORDER_BY_NEWNESS:
			self.log.sort(lambda x, y: cmp(y[0], x[0]))
		elif self.order == self.ORDER_BY_DATETIME:
			self.log.sort(lambda x, y: cmp(x[0], y[0]))
		elif self.order == self.ORDER_BY_VOTES:
			self.log.sort(lambda x, y: cmp((y[2], y[0]), (x[2], x[0])))
		# format logs
		self.formatted_log = []
		if not self.range == self.RANGE_LAST50BOTTLES:
			for datetime, mid, votes, agreements, channel, ifghost, script in self.log:
				self.formatted_log.append(["%s/%s/%s %s:%s:%s" % \
										   (datetime[0:4], datetime[4:6], datetime[6:8],
											datetime[8:10], datetime[10:12], datetime[12:14]),
										   ifghost, channel, "%2d" % votes, "%2d" % agreements,
										   script, mid])
		else:
			for i in range(len(self.log)):
				[datetime, mid, votes, agreements, channel, ifghost, script] = self.log[i]
				self.formatted_log.append(["%s/%s/%s %s:%s:%s" % \
										   (datetime[0:4], datetime[4:6], datetime[6:8],
											datetime[8:10], datetime[10:12], datetime[12:14]),
										   ifghost, channel, "%2d" % votes, "%2d" % agreements,
										   script, mid])
				if i >= 50:
					break

		return 0

	def set_channel(self, channel):
		self.channel = channel

	def get_channel(self):
		return self.channel

	def set_ghost(self, ghost):
		self.ghost = ghost

	def get_ghost(self):
		return self.ghost

	def set_range(self, range, range_start=None, range_end=None):
		self.range = range
		if range == self.RANGE_SPECIFIED:
			self.range_start = range_start
			self.range_end   = range_end

	def get_range(self):
		return self.range

	def set_order(self, order):
		self.order = order

	def get_order(self):
		return self.order

	def get_formatted_logs(self):
		if self.disabled:
			return None
		return self.formatted_log

	def get_selected_channels(self):
		return self.selected_channels

	def get_mid(self, index):
		if self.disabled or index >= len(self.log):
			return None
		return self.log[index][1]

	def get_script(self, index):
		if self.disabled or index >= len(self.log):
			return None
		return self.log[index][6]

class LogWindow:
	def __init__(self, app, logger, vote, agree):
		self.app = app
		self.logger = logger

		self.vote = vote
		self.agree = agree

		self.labels = {
			LogManager.ALL_CHANNELS: unicode(_("All"), "utf-8"),
			LogManager.RANGE_TODAY: unicode(_("Today"), "utf-8"),
			LogManager.RANGE_YESTERDAY: unicode(_("Yesterday"), "utf-8"),
			LogManager.RANGE_LAST3DAYS: unicode(_("Last 3 days"), "utf-8"),
			LogManager.RANGE_LAST7DAYS: unicode(_("Last a week"), "utf-8"),
			LogManager.RANGE_SPECIFIED: unicode(_("Specified"), "utf-8"),
			LogManager.RANGE_ALL: unicode(_("All"), "utf-8"),
			LogManager.RANGE_LAST50BOTTLES: unicode(_("Last 50 bottles"), "utf-8"),
			LogManager.ORDER_BY_NEWNESS: unicode(_("By newness"), "utf-8"),
			LogManager.ORDER_BY_DATETIME: unicode(_("By datetime"), "utf-8"),
			LogManager.ORDER_BY_VOTES: unicode(_("By votes"), "utf-8"),
			}

		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_size_request(450,300)

		# create GUI
		self.msg = {
			"close":  unicode(_("Close"), "utf-8"),
			"copy to main editor":   unicode(_("Copy to main editor"), "utf-8"),

			"view":		   [unicode(_("View(_V)"), "utf-8"),
							unicode(_("View(V)"), "utf-8")],
			"update":	   unicode(_("Update"), "utf-8"),
			"range":	   [unicode(_("Range(_R)"), "utf-8"),
							unicode(_("Range(R)"), "utf-8")],
			"style":	   [unicode(_("Style(_S)"), "utf-8"),
							unicode(_("Style(S)"), "utf-8")],
			"talk style":  unicode(_("Talk style"), "utf-8"),
			"script style":unicode(_("Script style"), "utf-8"),
			"linefeed":	   unicode(_("with linefeed"), "utf-8"),
			}

		menu_items = (
			( "/%s" % self.app.msg["file"][0], None, None, 0, "<Branch>" ),
			( "/%s/%s" % (self.app.msg["file"][1], self.msg["close"]), "<control>q", self.close, 0, None ),

			( "/%s" % self.app.msg["edit"][0], None, None, 0, "<Branch>" ),
			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy to main editor"]), "<control>c", self.edit_copy, 0, None ),
			( "/%s/sep" % self.app.msg["edit"][1], None, None, 0, "<Separator>" ),
			( "/%s/%s" % (self.app.msg["edit"][1], self.app.msg["pref"][0]), None, self.app.open_preference, 0, None ),

			( "/%s" % self.msg["view"][0], None, None, 0, "<Branch>" ),
			( "/%s/%s" % (self.msg["view"][1], self.msg["update"]), "<control>u", self.update, 0, None ),
			( "/%s/sep1" % self.msg["view"][1], None, None, 0, "<Separator>" ),

			( "/%s/%s" % (self.msg["view"][1], self.msg["range"][0]), None, None, 0, "<Branch>" ),
			( "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_LAST50BOTTLES],
			  None, self.initialize, LogManager.RANGE_LAST50BOTTLES, "<RadioItem>" ),
			( "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_TODAY],
			  None, self.initialize, LogManager.RANGE_TODAY,
			  "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_LAST50BOTTLES] ),
			( "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_YESTERDAY],
			  None, self.initialize, LogManager.RANGE_YESTERDAY,
			  "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_TODAY]),
			( "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_LAST3DAYS],
			  None, self.initialize, LogManager.RANGE_LAST3DAYS,
			  "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_YESTERDAY]),
			( "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_LAST7DAYS],
			  None, self.initialize, LogManager.RANGE_LAST7DAYS,
			  "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_LAST3DAYS]),
			( "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_ALL],
			  None, self.initialize, LogManager.RANGE_ALL,
			  "/%s/%s/" % (self.msg["view"][1], self.msg["range"][1]) + self.labels[LogManager.RANGE_LAST7DAYS]),

			( "/%s/%s" % (self.msg["view"][1], self.msg["style"][0]), None, None, 0, "<Branch>" ),
			( "/%s/%s/%s" % (self.msg["view"][1], self.msg["style"][1], self.msg["talk style"]),
			  None, self.set_style, textmanager.TextManager.STYLE_TALK, "<RadioItem>" ),
			( "/%s/%s/%s" % (self.msg["view"][1], self.msg["style"][1], self.msg["script style"]),
			  None, self.set_style, textmanager.TextManager.STYLE_SCRIPT,
			  "/%s/%s/%s" % (self.msg["view"][1],
							 self.msg["style"][1],
							 self.msg["talk style"])),
			( "/%s/%s/%s(%s)" % (self.msg["view"][1], self.msg["style"][1], self.msg["script style"], self.msg["linefeed"]),
			  None, self.set_style, textmanager.TextManager.STYLE_SCRIPT_WITH_LINEFEED,
			  "/%s/%s/%s" % (self.msg["view"][1],
							 self.msg["style"][1],
							 self.msg["script style"])),

			( "/%s" % self.app.msg["help"][0], None, None, 0, "<LastBranch>" ),
			( "/%s/%s" % (self.app.msg["help"][1], self.app.msg["about"]), None, self.app.about, 0, None ),
			)

		self.window.connect("delete_event", self.close)
		self.window.set_title(unicode(_("GBottler"), "utf-8") + ":" + \
							  unicode(_("Logs"), "utf-8"))

		self.main_vbox = gtk.VBox(gtk.FALSE, 0)
		self.window.add(self.main_vbox)
		self.main_vbox.show()

		accel_group = gtk.AccelGroup()

		self.item_factory = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)
		self.item_factory.create_items(menu_items)

		self.window.add_accel_group(accel_group)

		self.menu_list = {}
		for m in menu_items:
			key = re.sub("_", "", m[0])
			self.menu_list[key] = self.item_factory.get_item(key)

		self.menubar = self.item_factory.get_widget("<main>")
		self.main_vbox.pack_start(self.menubar, gtk.FALSE, gtk.TRUE, 0)
		self.menubar.show()
		# End of creating Menubar

		self.hbox = gtk.HBox()
		self.tooltips = gtk.Tooltips()

		self.pb = self.app.get_icon_button(gtk.STOCK_REFRESH,
										   gtk.ICON_SIZE_BUTTON)
		#								   label=unicode(_("Play"), "utf-8"))
		self.pb.connect("clicked", self.play, None)
		self.tooltips.set_tip(self.pb, unicode(_("Play"), "utf-8"))
		self.hbox.pack_start(self.pb, gtk.FALSE, gtk.TRUE, 0)

		self.vb = self.app.get_icon_button(gtk.STOCK_YES,
										   gtk.ICON_SIZE_BUTTON)
		#								   label=unicode(_("Vote"), "utf-8"))
		self.vb.connect("clicked", self.voting, None)
		self.vb.set_sensitive(gtk.FALSE)
		self.tooltips.set_tip(self.vb, unicode(_("Vote"), "utf-8"))
		self.hbox.pack_start(self.vb, gtk.FALSE, gtk.TRUE, 0)

		self.ab = self.app.get_icon_button(gtk.STOCK_NO,
										   gtk.ICON_SIZE_BUTTON)
		#								   label=unicode(_("Agree"), "utf-8"))
		self.ab.connect("clicked", self.agreeing, None)
		self.ab.set_sensitive(gtk.FALSE)
		self.tooltips.set_tip(self.ab, unicode(_("Agree"), "utf-8"))
		self.hbox.pack_start(self.ab, gtk.FALSE, gtk.TRUE, 0)

		self.l = gtk.Label("  " + unicode(_("Range"), "utf-8") + ": ")
		self.range_l = gtk.Label(unicode(_("Last 50 bottles"), "utf-8"))
		self.r_e_box = gtk.EventBox()
		self.r_e_box.set_events(gtk.gdk.BUTTON_PRESS_MASK)
		self.r_e_box.add(self.range_l)
		self.hbox.pack_start(self.l, gtk.FALSE, gtk.TRUE, 0)
		self.hbox.pack_start(self.r_e_box, gtk.FALSE, gtk.TRUE, 0)
		self.l.show()
		self.range_l.show()
		self.r_e_box.show()
		self.r_e_box.connect("button-press-event", self.button_popup,
							 self.item_factory.get_widget("/%s/%s" % (self.msg["view"][1],
																	  self.msg["range"][1])))

		self.m = gtk.Label("  " + unicode(_("Style"), "utf-8") + ": ")
		self.style_l = gtk.Label(self.msg['talk style'])
		self.s_e_box = gtk.EventBox()
		self.s_e_box.set_events(gtk.gdk.BUTTON_PRESS_MASK)
		self.s_e_box.add(self.style_l)
		self.hbox.pack_start(self.m, gtk.FALSE, gtk.TRUE, 0)
		self.hbox.pack_start(self.s_e_box, gtk.FALSE, gtk.TRUE, 0)
		self.m.show()
		self.style_l.show()
		self.s_e_box.show()
		self.s_e_box.connect("button-press-event", self.button_popup,
							 self.item_factory.get_widget("/%s/%s" % (self.msg["view"][1],
																	  self.msg["style"][1])))

		self.main_vbox.pack_start(self.hbox, gtk.FALSE, gtk.TRUE, 0)
		self.hbox.show()

		self.sw1 = gtk.ScrolledWindow()
		self.sw1.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.cols = [
			[unicode(_("Datetime"), "utf-8"), gtk.SORT_DESCENDING],
			[unicode(_("Ghost"), "utf-8"), gtk.SORT_DESCENDING],
			[unicode(_("Channel"), "utf-8"), gtk.SORT_DESCENDING],
			[unicode(_("Votes"), "utf-8"), gtk.SORT_DESCENDING],
			[unicode(_("Agrees"), "utf-8"), gtk.SORT_DESCENDING],
			[unicode(_("Script"), "utf-8"), gtk.SORT_DESCENDING],
			]
		self.entries = gtk.CList(len(self.cols), [ self.cols[i][0] for i in range(len(self.cols)) ])
		self.entries.set_shadow_type(gtk.SHADOW_IN)
		self.entries.set_selection_mode(gtk.SELECTION_BROWSE)
		self.entries.set_reorderable(gtk.TRUE)
		self.entries.set_size_request(450, 150)

		self.main_text = gtk.TextView()
		self.main_text.set_cursor_visible(gtk.FALSE)
		self.main_text.set_editable(gtk.FALSE)
		self.main_text.set_wrap_mode(gtk.WRAP_CHAR)
		self.main_text.set_border_width(0)
		self.main_text.show()

		self.textmanager = textmanager.TextManager(self, self.main_text, BottleParser('loose'))
		self.monitor = self.main_text.get_buffer()

		self.entries.connect("select-row", self.select)

		self.entries.set_column_width(0, 100)
		self.entries.set_column_auto_resize(1, gtk.TRUE)
		self.sw1.add(self.entries)
		self.sw1.show()
		self.entries.show()

		for i in range(len(self.cols)):
			self.entries.get_column_widget(i).get_parent().connect("clicked", self.sort, i)

		p_menu = (
			( "/sep1", None, None, 0, "<Separator>" ),
			( "/%s" % unicode(_("Copy to main editor")), "<control>c", self.edit_copy, 0, None ),
			( "/%s" % unicode(_("Play")), "<control>p", self.play, 0, None ),
			( "/%s" % unicode(_("Vote")), "<control><alt>v", self.vote, 0, None ),
			( "/%s" % unicode(_("Agree")), "<control><alt>a", self.agree, 0, None ),
			)
		self.p_ifact = gtk.ItemFactory(gtk.Menu, "<main>", accel_group)
		self.p_ifact.create_items(p_menu)
		self.popup_menu = self.p_ifact.get_widget("<main>")
		self.entries.connect("button-press-event", self.popup)
		self.entries.connect("focus-out-event", lambda x, y: self.entries.grab_focus())
		self.entries.connect("key-press-event", self.keypress)

		self.vi = self.p_ifact.get_item("/%s" % unicode(_("Vote"), "utf-8"))
		self.ai = self.p_ifact.get_item("/%s" % unicode(_("Agree"), "utf-8"))

		self.sw2 = gtk.ScrolledWindow()
		self.sw2.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.sw2.set_border_width(0)
		self.sw2.show()

		self.sw2.add(self.main_text)

		self.frame = gtk.Frame()
		self.frame.set_shadow_type(gtk.SHADOW_IN)
		self.frame.set_border_width(0)
		self.frame.add(self.sw2)
		self.frame.show()

		self.pane = gtk.VPaned()
		self.pane.pack1(self.sw1)
		self.pane.pack2(self.frame)
		self.pane.show()
		self.main_vbox.pack_start(self.pane)

		# initialize
		#self.view_style = self.STYLE_TALK
		self.current_sort = 0
		self.initialize(None, self.window)

	def keypress(self, widget, event):
		if event.state == 1: # shift
			if event.keyval == 65362: # up
				self.sw2.emit('scroll-child', gtk.SCROLL_PAGE_UP, gtk.FALSE)
			elif event.keyval == 65364: # down
				self.sw2.emit('scroll-child', gtk.SCROLL_PAGE_DOWN, gtk.FALSE)

		elif event.keyval == 65293: # enter
			self.play(widget, event)

	def set_style(self, data, widget):
		if data == textmanager.TextManager.STYLE_TALK:
			self.style_l.set_text(self.msg['talk style'])
		elif data == textmanager.TextManager.STYLE_SCRIPT:
			self.style_l.set_text(self.msg['script style'])
		elif data == textmanager.TextManager.STYLE_SCRIPT_WITH_LINEFEED:
			self.style_l.set_text("%s(%s)" % (self.msg['script style'], self.msg['linefeed']))

		#self.view_style = data
		self.textmanager.set_style(data)
		self.select(None, self.entries.selection[0], None, None)

	def select(self, clist, row, col, event):
		self.monitor.set_text("")
		self.textmanager.insert_with_color_from(unicode(self.entries.get_text(row, 5), "utf-8"))

	def sort(self, widget, colnum):
		if type(widget) is gtk.Button:
			self.cols[colnum][1] = not self.cols[colnum][1]

		self.entries.freeze()
		self.entries.set_sort_type(self.cols[colnum][1])
		self.entries.set_sort_column(colnum)
		self.entries.sort()
		self.entries.thaw()
		self.entries.grab_focus()
		self.current_sort = colnum

		if self.entries.selection:
			self.entries.moveto(self.entries.selection[0], colnum, 0.5, 0)
			self.set_focus_row(self.entries.selection[0], self.entries.rows)

	def popup(self, widget, event):
		sel = self.entries.get_selection_info(int(event.x), int(event.y))
		if sel:
			self.set_focus_row(sel[0], self.entries.rows)
			if event.button == 3:
				self.popup_menu.popup(None, None, None, event.button, 0)
			elif event.type == 5:
				self.play(widget, event)

	def button_popup(self, widget, event, sender):
		if event.button in [1, 3]:
			sender.popup(None, None, None, event.button, 0)

	def open(self, widget, data):
		self.window.show()

	def close(self, widget=None, e=None, data=None):
		self.window.hide()
		return gtk.TRUE

	def update(self, data, widget):
		range = self.logger.get_range()
		if range:
			self.initialize(range, widget)
		else:
			self.initialize(None, widget)

	def initialize(self, changed, widget):
		# Bad /-P
		current_row = None
		if self.entries.selection:
			current_row = self.entries.get_row_data(self.entries.selection[0])

		self.monitor.set_text("")

		if changed in [None, 0]:
			self.logger.set_channel(LogManager.ALL_CHANNELS)
			self.logger.set_range(LogManager.RANGE_LAST50BOTTLES)
			self.range_l.set_text(self.labels[LogManager.RANGE_LAST50BOTTLES])

		elif type(changed) == StringType:
			self.logger.set_channel(changed)

		elif changed <= 100:
			self.logger.set_range(changed)
			self.range_l.set_text(self.labels[changed])

		error = self.logger.reload()
		if error:
			return

 		loglist = self.logger.get_formatted_logs()
		self.entries.freeze()
		self.entries.clear()
		for log in loglist:
			mid = log[-1]
			del log[-1]
			tuple(log)
			n = self.entries.append(log)
			self.entries.set_row_data(n, mid)
		self.sort(widget, self.current_sort)

		# Bad /-P
		if not current_row == None:
			for i in range(self.entries.rows):
				if self.entries.get_row_data(i) == current_row:
					current_row = i
					break
			if type(current_row) is int:
				self.set_focus_row(current_row, self.entries.rows)
				self.entries.moveto(current_row, 0, 0.5, 0)

		self.entries.columns_autosize()
		self.entries.thaw()
		self.entries.grab_focus()

		if self.entries.rows <= 0:
			self.pb.set_sensitive(gtk.FALSE)
		else:
			self.pb.set_sensitive(gtk.TRUE)

	def set_focus_row(self, row, row_count):
		if row_count == 0:
			return
		if row_count == 1:
			r = 0
		else:
			r = min((row+0.00001)/(row_count-1),1.0)
		self.entries.emit("scroll-vertical", gtk.SCROLL_JUMP, r)

	def play(self, widget, data):
		script = unicode(self.entries.get_text(self.entries.selection[0], 5), 'utf-8')
		ifghost = unicode(self.entries.get_text(self.entries.selection[0], 1), 'utf-8')
		channel = unicode(self.entries.get_text(self.entries.selection[0], 2), 'utf-8')
		if script is None or ifghost is None:
			return
		self.app.send_local_message(channel, ifghost, script)

	def voting(self, widget, data):
		mid = self.entries.get_row_data(self.entries.selection[0])
		if mid is None:
			return

		d = gtk.Dialog(unicode(_("Really?"), "utf-8"), parent=self.window,
					   flags=gtk.DIALOG_NO_SEPARATOR|gtk.DIALOG_MODAL)
		d.connect("destroy", lambda w: w.destroy)
		d.set_border_width(10)
		l = gtk.Label(unicode(_("Vote this bottle?"), "utf-8"))
		l.set_justify(gtk.JUSTIFY_RIGHT)
		l.show()
		d.vbox.add(l)
		d.add_button(unicode(_("OK"), "utf-8"), False)
		d.add_button(unicode(_("Cancel"), "utf-8"), True)
		d.show()
		cancel = d.run()
		d.destroy()

		if cancel:
			return
		self.app.vote_message(mid, self.vote)

	def agreeing(self, widget, data):
		mid = self.entries.get_row_data(self.entries.selection[0])
		if mid is None:
			return

		d = gtk.Dialog(unicode(_("Really?"), "utf-8"), parent=self.window,
					   flags=gtk.DIALOG_NO_SEPARATOR|gtk.DIALOG_MODAL)
		d.connect("destroy", lambda w: w.destroy)
		d.set_border_width(10)
		l = gtk.Label(unicode(_("Agree this bottle?"), "utf-8"))
		l.set_justify(gtk.JUSTIFY_RIGHT)
		l.show()
		d.vbox.add(l)
		d.add_button(unicode(_("OK"), "utf-8"), False)
		d.add_button(unicode(_("Cancel"), "utf-8"), True)
		d.show()
		cancel = d.run()
		d.destroy()

		if cancel:
			return
		self.app.vote_message(mid, self.agree)

	def edit_copy(self, widget, data):
		if not self.entries.selection:
			return
		script = self.entries.get_text(self.entries.selection[0], 5)
		if not script:
			return

		self.app.insert(script)

	def help(self, event=None):
		pass
